@@ -1,6 +1,7 @@
 """
-A DOMNode is a base class for any object within the Textual Document Object Model,
+The module contains `DOMNode`, the base class for any object within the Textual Document Object Model,
 which includes all Widgets, Screens, and Apps.
+
 """
 
 from __future__ import annotations
@@ -9,6 +10,7 @@ import re
 import threading
 from functools import lru_cache, partial
 from inspect import getfile
+from operator import attrgetter
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -28,38 +30,43 @@ from rich.style import Style
 from rich.text import Text
 from rich.tree import Tree
 
-from ._context import NoActiveAppError, active_message_pump
-from ._node_list import NodeList
-from ._types import WatchCallbackType
-from ._worker_manager import WorkerManager
-from .binding import Binding, BindingType, _Bindings
-from .color import BLACK, WHITE, Color
-from .css._error_tools import friendly_list
-from .css.constants import VALID_DISPLAY, VALID_VISIBILITY
-from .css.errors import DeclarationError, StyleValueError
-from .css.parse import parse_declarations
-from .css.styles import RenderStyles, Styles
-from .css.tokenize import IDENTIFIER
-from .message_pump import MessagePump
-from .reactive import Reactive, ReactiveError, _watch
-from .timer import Timer
-from .walk import walk_breadth_first, walk_depth_first
+from textual._context import NoActiveAppError, active_message_pump
+from textual._node_list import NodeList
+from textual._types import WatchCallbackType
+from textual.binding import Binding, BindingsMap, BindingType
+from textual.cache import LRUCache
+from textual.color import BLACK, WHITE, Color
+from textual.css._error_tools import friendly_list
+from textual.css.constants import VALID_DISPLAY, VALID_VISIBILITY
+from textual.css.errors import DeclarationError, StyleValueError
+from textual.css.match import match
+from textual.css.parse import parse_declarations, parse_selectors
+from textual.css.query import InvalidQueryFormat, NoMatches, TooManyMatches
+from textual.css.styles import RenderStyles, Styles
+from textual.css.tokenize import IDENTIFIER
+from textual.css.tokenizer import TokenError
+from textual.message_pump import MessagePump
+from textual.reactive import Reactive, ReactiveError, _Mutated, _watch
+from textual.style import Style as VisualStyle
+from textual.timer import Timer
+from textual.walk import walk_breadth_first, walk_depth_first
+from textual.worker_manager import WorkerManager
 
 if TYPE_CHECKING:
     from typing_extensions import Self, TypeAlias
     from _typeshed import SupportsRichComparison
 
     from rich.console import RenderableType
-    from .app import App
-    from .css.query import DOMQuery, QueryType
-    from .css.types import CSSLocation
-    from .message import Message
-    from .screen import Screen
-    from .widget import Widget
-    from .worker import Worker, WorkType, ResultType
+    from textual.app import App
+    from textual.css.query import DOMQuery, QueryType
+    from textual.css.types import CSSLocation
+    from textual.message import Message
+    from textual.screen import Screen
+    from textual.widget import Widget
+    from textual.worker import Worker, WorkType, ResultType
 
     # Unused & ignored imports are needed for the docs to link to these objects:
-    from .css.query import NoMatches, TooManyMatches, WrongType  # type: ignore  # noqa: F401
+    from textual.css.query import WrongType  # type: ignore  # noqa: F401
 
 from typing_extensions import Literal
 
@@ -73,6 +80,10 @@ WalkMethod: TypeAlias = Literal["depth", "breadth"]
 ReactiveType = TypeVar("ReactiveType")
 
 
+QueryOneCacheKey: TypeAlias = "tuple[int, str, Type[Widget] | None]"
+"""The key used to cache query_one results."""
+
+
 class BadIdentifier(Exception):
     """Exception raised if you supply a `id` attribute or class name in the wrong format."""
 
@@ -125,22 +136,28 @@ class _ClassesDescriptor:
 class DOMNode(MessagePump):
     """The base class for object that can be in the Textual DOM (App and Widget)"""
 
-    # CSS defaults
     DEFAULT_CSS: ClassVar[str] = ""
+    """Default TCSS."""
 
-    # Default classes argument if not supplied
-    DEFAULT_CLASSES: str = ""
+    DEFAULT_CLASSES: ClassVar[str] = ""
+    """Default classes argument if not supplied."""
 
-    # Virtual DOM nodes
     COMPONENT_CLASSES: ClassVar[set[str]] = set()
+    """Virtual DOM nodes, used to expose styles to line API widgets."""
+
+    BINDING_GROUP_TITLE: str | None = None
+    """Title of widget used where bindings are displayed (such as in the key panel)."""
 
-    # Mapping of key bindings
     BINDINGS: ClassVar[list[BindingType]] = []
+    """A list of key bindings."""
 
     # Indicates if the CSS should be automatically scoped
     SCOPED_CSS: ClassVar[bool] = True
     """Should default css be limited to the widget type?"""
 
+    HELP: ClassVar[str | None] = None
+    """Optional help text shown in help panel (Markdown format)."""
+
     # True if this node inherits the CSS from the base class.
     _inherit_css: ClassVar[bool] = True
 
@@ -157,7 +174,7 @@ class DOMNode(MessagePump):
     _css_type_name: str = ""
 
     # Generated list of bindings
-    _merged_bindings: ClassVar[_Bindings | None] = None
+    _merged_bindings: ClassVar[BindingsMap | None] = None
 
     _reactives: ClassVar[dict[str, Reactive]]
 
@@ -166,6 +183,9 @@ class DOMNode(MessagePump):
     # Names of potential computed reactives
     _computes: ClassVar[frozenset[str]]
 
+    _PSEUDO_CLASSES: ClassVar[dict[str, Callable[[object], bool]]] = {}
+    """Pseudo class checks."""
+
     def __init__(
         self,
         *,
@@ -177,13 +197,14 @@ class DOMNode(MessagePump):
         self._name = name
         self._id = None
         if id is not None:
-            self.id = id
+            check_identifiers("id", id)
+            self._id = id
 
         _classes = classes.split() if classes else []
         check_identifiers("class name", *_classes)
         self._classes.update(_classes)
 
-        self._nodes: NodeList = NodeList()
+        self._nodes: NodeList = NodeList(self)
         self._css_styles: Styles = Styles(self)
         self._inline_styles: Styles = Styles(self)
         self.styles: RenderStyles = RenderStyles(
@@ -196,18 +217,35 @@ class DOMNode(MessagePump):
         self._auto_refresh_timer: Timer | None = None
         self._css_types = {cls.__name__ for cls in self._css_bases(self.__class__)}
         self._bindings = (
-            _Bindings()
+            BindingsMap()
             if self._merged_bindings is None
             else self._merged_bindings.copy()
         )
         self._has_hover_style: bool = False
         self._has_focus_within: bool = False
+        self._has_order_style: bool = False
+        """The node has an ordered dependent pseudo-style (`:odd`, `:even`, `:first-of-type`, `:last-of-type`)"""
+        self._has_odd_or_even: bool = False
+        """The node has the pseudo class `odd` or `even`."""
         self._reactive_connect: (
-            dict[str, tuple[MessagePump, Reactive | object]] | None
+            dict[str, tuple[MessagePump, Reactive[object] | object]] | None
         ) = None
+        self._pruning = False
+        self._query_one_cache: LRUCache[QueryOneCacheKey, DOMNode] = LRUCache(1024)
 
         super().__init__()
 
+    def _get_dom_base(self) -> DOMNode:
+        """Get the DOM base node (typically self).
+
+        All DOM queries on this node will use the return value as the root node.
+        This method allows the App to query the default screen, and not the active screen.
+
+        Returns:
+            DOMNode.
+        """
+        return self
+
     def set_reactive(
         self, reactive: Reactive[ReactiveType], value: ReactiveType
     ) -> None:
@@ -215,25 +253,51 @@ class DOMNode(MessagePump):
 
         Example:
             ```python
-            self.set_reactive(App.dark_mode, True)
+            self.set_reactive(App.theme, "textual-light")
             ```
 
         Args:
-            name: Name of reactive attribute.
+            reactive: A reactive property (use the class scope syntax, i.e. `MyClass.my_reactive`).
             value: New value of reactive.
 
         Raises:
             AttributeError: If the first argument is not a reactive.
         """
+        name = reactive.name
         if not isinstance(reactive, Reactive):
-            raise TypeError(
-                "A Reactive class is required; for example: MyApp.dark_mode"
-            )
-        if reactive.name not in self._reactives:
+            raise TypeError("A Reactive class is required; for example: MyApp.theme")
+        if name not in self._reactives:
             raise AttributeError(
-                "No reactive called {name!r}; Have you called super().__init__(...) in the {self.__class__.__name__} constructor?"
+                f"No reactive called {name!r}; Have you called super().__init__(...) in the {self.__class__.__name__} constructor?"
             )
-        setattr(self, f"_reactive_{reactive.name}", value)
+        setattr(self, f"_reactive_{name}", value)
+
+    def mutate_reactive(self, reactive: Reactive[ReactiveType]) -> None:
+        """Force an update to a mutable reactive.
+
+        Example:
+            ```python
+            self.reactive_name_list.append("Jessica")
+            self.mutate_reactive(MyClass.reactive_name_list)
+            ```
+
+        Textual will automatically detect when a reactive is set to a new value, but it is unable
+        to detect if a value is _mutated_ (such as updating a list, dict, or attribute of an object).
+        If you do wish to use a collection or other mutable object in a reactive, then you can call
+        this method after your reactive is updated. This will ensure that all the reactive _superpowers_
+        work.
+
+        !!! note
+
+            This method will cause watchers to be called, even if the value hasn't changed.
+
+        Args:
+            reactive: A reactive property (use the class scope syntax, i.e. `MyClass.my_reactive`).
+        """
+
+        internal_name = f"_reactive_{reactive.name}"
+        value = getattr(self, internal_name)
+        reactive._set(self, value, always=True)
 
     def data_bind(
         self,
@@ -308,7 +372,8 @@ class DOMNode(MessagePump):
                     """Set bound data."""
                     _rich_traceback_omit = True
                     Reactive._initialize_object(self)
-                    setattr(self, variable_name, value)
+                    # Wrap the value in `_Mutated` so the setter knows to invoke watchers etc.
+                    setattr(self, variable_name, _Mutated(value))
 
                 return setter
 
@@ -382,7 +447,7 @@ class DOMNode(MessagePump):
             self._auto_refresh_timer = None
         if interval is not None:
             self._auto_refresh_timer = self.set_interval(
-                interval, self._automatic_refresh, name=f"auto refresh {self!r}"
+                interval, self.automatic_refresh, name=f"auto refresh {self!r}"
             )
         self._auto_refresh = interval
 
@@ -444,9 +509,21 @@ class DOMNode(MessagePump):
         """Is the node a modal?"""
         return False
 
-    def _automatic_refresh(self) -> None:
-        """Perform an automatic refresh (set with auto_refresh property)."""
-        self.refresh()
+    @property
+    def is_on_screen(self) -> bool:
+        """Check if the node was displayed in the last screen update."""
+        return False
+
+    def automatic_refresh(self) -> None:
+        """Perform an automatic refresh.
+
+        This method is called when you set the `auto_refresh` attribute.
+        You could implement this method if you want to perform additional work
+        during an automatic refresh.
+
+        """
+        if self.is_on_screen:
+            self.refresh()
 
     def __init_subclass__(
         cls,
@@ -484,11 +561,11 @@ class DOMNode(MessagePump):
             ]
         )
 
-    def get_component_styles(self, name: str) -> RenderStyles:
+    def get_component_styles(self, *names: str) -> RenderStyles:
         """Get a "component" styles object (must be defined in COMPONENT_CLASSES classvar).
 
         Args:
-            name: Name of the component.
+            names: Names of the components.
 
         Raises:
             KeyError: If the component class doesn't exist.
@@ -496,9 +573,18 @@ class DOMNode(MessagePump):
         Returns:
             A Styles object.
         """
-        if name not in self._component_styles:
-            raise KeyError(f"No {name!r} key in COMPONENT_CLASSES")
-        styles = self._component_styles[name]
+
+        styles = RenderStyles(self, Styles(), Styles())
+
+        for name in names:
+            if name not in self._component_styles:
+                raise KeyError(f"No {name!r} key in COMPONENT_CLASSES")
+            component_styles = self._component_styles[name]
+            styles.node = component_styles.node
+            styles.base.merge(component_styles.base)
+            styles.inline.merge(component_styles.inline)
+            styles._updates += 1
+
         return styles
 
     def _post_mount(self):
@@ -544,27 +630,31 @@ class DOMNode(MessagePump):
         return classes
 
     @classmethod
-    def _merge_bindings(cls) -> _Bindings:
+    def _merge_bindings(cls) -> BindingsMap:
         """Merge bindings from base classes.
 
         Returns:
             Merged bindings.
         """
-        bindings: list[_Bindings] = []
+        bindings: list[BindingsMap] = []
 
         for base in reversed(cls.__mro__):
             if issubclass(base, DOMNode):
                 if not base._inherit_bindings:
                     bindings.clear()
                 bindings.append(
-                    _Bindings(
+                    BindingsMap(
                         base.__dict__.get("BINDINGS", []),
                     )
                 )
-        keys: dict[str, Binding] = {}
+
+        keys: dict[str, list[Binding]] = {}
         for bindings_ in bindings:
-            keys.update(bindings_.keys)
-        return _Bindings(keys.values())
+            for key, key_bindings in bindings_.key_to_bindings.items():
+                keys[key] = key_bindings
+
+        new_bindings = BindingsMap.from_keys(keys)
+        return new_bindings
 
     def _post_register(self, app: App) -> None:
         """Called when the widget is registered
@@ -664,11 +754,16 @@ class DOMNode(MessagePump):
         """
         # Get the node by looking up a chain of parents
         # Note that self.screen may not be the same as self.app.screen
-        from .screen import Screen
+        from textual.screen import Screen
 
         node: MessagePump | None = self
-        while node is not None and not isinstance(node, Screen):
-            node = node._parent
+        try:
+            while node is not None and not isinstance(node, Screen):
+                node = node._parent
+        except AttributeError:
+            raise RuntimeError(
+                "Widget is missing attributes; have you called the constructor in your widget class?"
+            ) from None
         if not isinstance(node, Screen):
             raise NoScreen("node has no screen")
         return node
@@ -689,7 +784,7 @@ class DOMNode(MessagePump):
             ValueError: If the ID has already been set.
         """
         check_identifiers("id", new_id)
-
+        self._nodes.updated()
         if self._id is not None:
             raise ValueError(
                 f"Node 'id' attribute may not be changed once set (current id={self._id!r})"
@@ -746,8 +841,7 @@ class DOMNode(MessagePump):
         append = result.append
 
         node: DOMNode = self
-        while isinstance(node._parent, DOMNode):
-            node = node._parent
+        while isinstance((node := node._parent), DOMNode):
             append(node)
         return result[::-1]
 
@@ -778,7 +872,9 @@ class DOMNode(MessagePump):
             my_widget.display = False  # Hide my_widget
             ```
         """
-        return self.styles.display != "none" and not (self._closing or self._closed)
+        return self.styles.display != "none" and not (
+            self._closing or self._closed or self._pruning
+        )
 
     @display.setter
     def display(self, new_val: bool | str) -> None:
@@ -882,7 +978,7 @@ class DOMNode(MessagePump):
         from rich.panel import Panel
         from rich.pretty import Pretty
 
-        from .widget import Widget
+        from textual.widget import Widget
 
         def render_info(node: DOMNode) -> Columns:
             """Render a node for the tree."""
@@ -939,6 +1035,12 @@ class DOMNode(MessagePump):
             node.styles.text_style for node in reversed(self.ancestors_with_self)
         )
 
+    @property
+    def selection_style(self) -> Style:
+        """The style of selected text."""
+        style = self.screen.get_component_rich_style("screen--selection")
+        return style
+
     @property
     def rich_style(self) -> Style:
         """Get a Rich Style object for this DOMNode.
@@ -954,16 +1056,21 @@ class DOMNode(MessagePump):
 
         for node in reversed(self.ancestors_with_self):
             styles = node.styles
+            has_rule = styles.has_rule
             opacity *= styles.opacity
-            if styles.has_rule("background"):
-                text_background = background + styles.background
-                background += styles.background.multiply_alpha(opacity)
+            if has_rule("background"):
+                text_background = background + styles.background.tint(
+                    styles.background_tint
+                )
+                background += (
+                    styles.background.tint(styles.background_tint)
+                ).multiply_alpha(opacity)
             else:
                 text_background = background
-            if styles.has_rule("color"):
+            if has_rule("color"):
                 color = styles.color
             style += styles.text_style
-            if styles.has_rule("auto_color") and styles.auto_color:
+            if has_rule("auto_color") and styles.auto_color:
                 color = text_background.get_contrast_text(color.a)
 
         style += Style.from_color(
@@ -972,10 +1079,28 @@ class DOMNode(MessagePump):
         )
         return style
 
+    def check_consume_key(self, key: str, character: str | None) -> bool:
+        """Check if the widget may consume the given key.
+
+        This should be implemented in widgets that handle [`Key`][textual.events.Key] events and
+        stop propagation (such as Input and TextArea).
+
+        Implementing this method will hide key bindings from the footer and key panel that would
+        be *consumed* by the focused widget.
+
+        Args:
+            key: A key identifier.
+            character: A character associated with the key, or `None` if there isn't one.
+
+        Returns:
+            `True` if the widget may capture the key in its `Key` event handler, or `False` if it won't.
+        """
+        return False
+
     def _get_title_style_information(
         self, background: Color
-    ) -> tuple[Color, Color, Style]:
-        """Get a Rich Style object for for titles.
+    ) -> tuple[Color, Color, VisualStyle]:
+        """Get a Visual Style object for for titles.
 
         Args:
             background: The background color.
@@ -983,6 +1108,7 @@ class DOMNode(MessagePump):
         Returns:
             A Rich style.
         """
+
         styles = self.styles
         if styles.auto_border_title_color:
             color = background.get_contrast_text(styles.border_title_color.a)
@@ -991,12 +1117,12 @@ class DOMNode(MessagePump):
         return (
             color,
             styles.border_title_background,
-            styles.border_title_style,
+            VisualStyle.from_rich_style(styles.border_title_style),
         )
 
     def _get_subtitle_style_information(
         self, background: Color
-    ) -> tuple[Color, Color, Style]:
+    ) -> tuple[Color, Color, VisualStyle]:
         """Get a Rich Style object for for titles.
 
         Args:
@@ -1013,7 +1139,7 @@ class DOMNode(MessagePump):
         return (
             color,
             styles.border_subtitle_background,
-            styles.border_subtitle_style,
+            VisualStyle.from_rich_style(styles.border_subtitle_style),
         )
 
     @property
@@ -1027,7 +1153,7 @@ class DOMNode(MessagePump):
         for node in reversed(self.ancestors_with_self):
             styles = node.styles
             base_background = background
-            background += styles.background
+            background += styles.background.tint(styles.background_tint)
         return (base_background, background)
 
     @property
@@ -1043,7 +1169,9 @@ class DOMNode(MessagePump):
             styles = node.styles
             base_background = background
             opacity *= styles.opacity
-            background += styles.background.multiply_alpha(opacity)
+            background += styles.background.tint(styles.background_tint).multiply_alpha(
+                opacity
+            )
         return (base_background, background)
 
     @property
@@ -1058,7 +1186,7 @@ class DOMNode(MessagePump):
         for node in reversed(self.ancestors_with_self):
             styles = node.styles
             base_background = background
-            background += styles.background
+            background += styles.background.tint(styles.background_tint)
             if styles.has_rule("color"):
                 base_color = color
                 if styles.auto_color:
@@ -1078,12 +1206,11 @@ class DOMNode(MessagePump):
         Returns:
             A list of nodes.
         """
-        nodes: list[MessagePump | None] = []
+        nodes: list[MessagePump | None] = [self]
         add_node = nodes.append
         node: MessagePump | None = self
-        while node is not None:
+        while (node := node._parent) is not None:
             add_node(node)
-            node = node._parent
         return cast("list[DOMNode]", nodes)
 
     @property
@@ -1093,7 +1220,12 @@ class DOMNode(MessagePump):
         Returns:
             A list of nodes.
         """
-        return self.ancestors_with_self[1:]
+        nodes: list[MessagePump | None] = []
+        add_node = nodes.append
+        node: MessagePump | None = self
+        while (node := node._parent) is not None:
+            add_node(node)
+        return cast("list[DOMNode]", nodes)
 
     @property
     def displayed_children(self) -> list[Widget]:
@@ -1102,7 +1234,7 @@ class DOMNode(MessagePump):
         Returns:
             A list of nodes.
         """
-        return [child for child in self._nodes if child.display]
+        return list(filter(attrgetter("display"), self._nodes))
 
     def watch(
         self,
@@ -1114,15 +1246,12 @@ class DOMNode(MessagePump):
         """Watches for modifications to reactive attributes on another object.
 
         Example:
-
-            Here's how you could detect when the app changes from dark to light mode (and vice versa).
-
             ```python
-            def on_dark_change(old_value:bool, new_value:bool):
-                # Called when app.dark changes.
-                print("App.dark when from {old_value} to {new_value}")
+            def on_theme_change(old_value:str, new_value:str) -> None:
+                # Called when app.theme changes.
+                print(f"App.theme went from {old_value} to {new_value}")
 
-            self.watch(self.app, "dark", self.on_dark_change, init=False)
+            self.watch(self.app, "theme", self.on_theme_change, init=False)
             ```
 
         Args:
@@ -1133,17 +1262,22 @@ class DOMNode(MessagePump):
         """
         _watch(self, obj, attribute_name, callback, init=init)
 
-    def get_pseudo_classes(self) -> Iterable[str]:
-        """Get any pseudo classes applicable to this Node, e.g. hover, focus.
+    def get_pseudo_classes(self) -> set[str]:
+        """Pseudo classes for a widget.
 
         Returns:
-            Iterable of strings, such as a generator.
+            Names of the pseudo classes.
         """
-        return ()
+
+        return {
+            name
+            for name, check_class in self._PSEUDO_CLASSES.items()
+            if check_class(self)
+        }
 
     def reset_styles(self) -> None:
         """Reset styles back to their initial state."""
-        from .widget import Widget
+        from textual.widget import Widget
 
         for node in self.walk_children(with_self=True):
             node._css_styles.reset()
@@ -1180,24 +1314,26 @@ class DOMNode(MessagePump):
 
     WalkType = TypeVar("WalkType", bound="DOMNode")
 
-    @overload
-    def walk_children(
-        self,
-        filter_type: type[WalkType],
-        *,
-        with_self: bool = False,
-        method: WalkMethod = "depth",
-        reverse: bool = False,
-    ) -> list[WalkType]: ...
+    if TYPE_CHECKING:
 
-    @overload
-    def walk_children(
-        self,
-        *,
-        with_self: bool = False,
-        method: WalkMethod = "depth",
-        reverse: bool = False,
-    ) -> list[DOMNode]: ...
+        @overload
+        def walk_children(
+            self,
+            filter_type: type[WalkType],
+            *,
+            with_self: bool = False,
+            method: WalkMethod = "depth",
+            reverse: bool = False,
+        ) -> list[WalkType]: ...
+
+        @overload
+        def walk_children(
+            self,
+            *,
+            with_self: bool = False,
+            method: WalkMethod = "depth",
+            reverse: bool = False,
+        ) -> list[DOMNode]: ...
 
     def walk_children(
         self,
@@ -1233,11 +1369,13 @@ class DOMNode(MessagePump):
             nodes.reverse()
         return cast("list[DOMNode]", nodes)
 
-    @overload
-    def query(self, selector: str | None) -> DOMQuery[Widget]: ...
+    if TYPE_CHECKING:
 
-    @overload
-    def query(self, selector: type[QueryType]) -> DOMQuery[QueryType]: ...
+        @overload
+        def query(self, selector: str | None = None) -> DOMQuery[Widget]: ...
+
+        @overload
+        def query(self, selector: type[QueryType]) -> DOMQuery[QueryType]: ...
 
     def query(
         self, selector: str | type[QueryType] | None = None
@@ -1250,22 +1388,59 @@ class DOMNode(MessagePump):
         Returns:
             A query object.
         """
-        from .css.query import DOMQuery, QueryType
-        from .widget import Widget
+        from textual.css.query import DOMQuery, QueryType
+        from textual.widget import Widget
 
+        node = self._get_dom_base()
         if isinstance(selector, str) or selector is None:
-            return DOMQuery[Widget](self, filter=selector)
+            return DOMQuery[Widget](node, filter=selector)
         else:
-            return DOMQuery[QueryType](self, filter=selector.__name__)
+            return DOMQuery[QueryType](node, filter=selector.__name__)
 
-    @overload
-    def query_one(self, selector: str) -> Widget: ...
+    if TYPE_CHECKING:
 
-    @overload
-    def query_one(self, selector: type[QueryType]) -> QueryType: ...
+        @overload
+        def query_children(self, selector: str | None = None) -> DOMQuery[Widget]: ...
 
-    @overload
-    def query_one(self, selector: str, expect_type: type[QueryType]) -> QueryType: ...
+        @overload
+        def query_children(self, selector: type[QueryType]) -> DOMQuery[QueryType]: ...
+
+    def query_children(
+        self, selector: str | type[QueryType] | None = None
+    ) -> DOMQuery[Widget] | DOMQuery[QueryType]:
+        """Query the DOM for the immediate children that match a selector or widget type.
+
+        Note that this will not return child widgets more than a single level deep.
+        If you want to a query to potentially match all children in the widget tree,
+        see [query][textual.dom.DOMNode.query].
+
+        Args:
+            selector: A CSS selector, widget type, or `None` for all nodes.
+
+        Returns:
+            A query object.
+        """
+        from textual.css.query import DOMQuery, QueryType
+        from textual.widget import Widget
+
+        node = self._get_dom_base()
+        if isinstance(selector, str) or selector is None:
+            return DOMQuery[Widget](node, deep=False, filter=selector)
+        else:
+            return DOMQuery[QueryType](node, deep=False, filter=selector.__name__)
+
+    if TYPE_CHECKING:
+
+        @overload
+        def query_one(self, selector: str) -> Widget: ...
+
+        @overload
+        def query_one(self, selector: type[QueryType]) -> QueryType: ...
+
+        @overload
+        def query_one(
+            self, selector: str, expect_type: type[QueryType]
+        ) -> QueryType: ...
 
     def query_one(
         self,
@@ -1281,21 +1456,176 @@ class DOMNode(MessagePump):
         Raises:
             WrongType: If the wrong type was found.
             NoMatches: If no node matches the query.
-            TooManyMatches: If there is more than one matching node in the query.
 
         Returns:
             A widget matching the selector.
         """
         _rich_traceback_omit = True
-        from .css.query import DOMQuery
+
+        base_node = self._get_dom_base()
 
         if isinstance(selector, str):
             query_selector = selector
         else:
             query_selector = selector.__name__
-        query: DOMQuery[Widget] = DOMQuery(self, filter=query_selector)
 
-        return query.only_one() if expect_type is None else query.only_one(expect_type)
+        try:
+            selector_set = parse_selectors(query_selector)
+        except TokenError:
+            raise InvalidQueryFormat(
+                f"Unable to parse {query_selector!r} as a query; check for syntax errors"
+            ) from None
+
+        if all(selectors.is_simple for selectors in selector_set):
+            cache_key = (base_node._nodes._updates, query_selector, expect_type)
+            cached_result = base_node._query_one_cache.get(cache_key)
+            if cached_result is not None:
+                return cached_result
+        else:
+            cache_key = None
+
+        for node in walk_depth_first(base_node, with_root=False):
+            if not match(selector_set, node):
+                continue
+            if expect_type is not None and not isinstance(node, expect_type):
+                continue
+            if cache_key is not None:
+                base_node._query_one_cache[cache_key] = node
+            return node
+
+        raise NoMatches(f"No nodes match {selector!r} on {self!r}")
+
+    if TYPE_CHECKING:
+
+        @overload
+        def query_exactly_one(self, selector: str) -> Widget: ...
+
+        @overload
+        def query_exactly_one(self, selector: type[QueryType]) -> QueryType: ...
+
+        @overload
+        def query_exactly_one(
+            self, selector: str, expect_type: type[QueryType]
+        ) -> QueryType: ...
+
+    def query_exactly_one(
+        self,
+        selector: str | type[QueryType],
+        expect_type: type[QueryType] | None = None,
+    ) -> QueryType | Widget:
+        """Get a widget from this widget's children that matches a selector or widget type.
+
+        !!! Note
+            This method is similar to [query_one][textual.dom.DOMNode.query_one].
+            The only difference is that it will raise `TooManyMatches` if there is more than a single match.
+
+        Args:
+            selector: A selector or widget type.
+            expect_type: Require the object be of the supplied type, or None for any type.
+
+        Raises:
+            WrongType: If the wrong type was found.
+            NoMatches: If no node matches the query.
+            TooManyMatches: If there is more than one matching node in the query (and `exactly_one==True`).
+
+        Returns:
+            A widget matching the selector.
+        """
+        _rich_traceback_omit = True
+
+        base_node = self._get_dom_base()
+
+        if isinstance(selector, str):
+            query_selector = selector
+        else:
+            query_selector = selector.__name__
+
+        try:
+            selector_set = parse_selectors(query_selector)
+        except TokenError:
+            raise InvalidQueryFormat(
+                f"Unable to parse {query_selector!r} as a query; check for syntax errors"
+            ) from None
+
+        if all(selectors.is_simple for selectors in selector_set):
+            cache_key = (base_node._nodes._updates, query_selector, expect_type)
+            cached_result = base_node._query_one_cache.get(cache_key)
+            if cached_result is not None:
+                return cached_result
+        else:
+            cache_key = None
+
+        children = walk_depth_first(base_node, with_root=False)
+        iter_children = iter(children)
+        for node in iter_children:
+            if not match(selector_set, node):
+                continue
+            if expect_type is not None and not isinstance(node, expect_type):
+                continue
+            for later_node in iter_children:
+                if match(selector_set, later_node):
+                    if expect_type is not None and not isinstance(node, expect_type):
+                        continue
+                    raise TooManyMatches(
+                        "Call to query_one resulted in more than one matched node"
+                    )
+            if cache_key is not None:
+                base_node._query_one_cache[cache_key] = node
+            return node
+
+        raise NoMatches(f"No nodes match {selector!r} on {self!r}")
+
+    if TYPE_CHECKING:
+
+        @overload
+        def query_ancestor(self, selector: str) -> DOMNode: ...
+
+        @overload
+        def query_ancestor(self, selector: type[QueryType]) -> QueryType: ...
+
+        @overload
+        def query_ancestor(
+            self, selector: str, expect_type: type[QueryType]
+        ) -> QueryType: ...
+
+    def query_ancestor(
+        self,
+        selector: str | type[QueryType],
+        expect_type: type[QueryType] | None = None,
+    ) -> DOMNode | None:
+        """Get an ancestor which matches a query.
+
+        Args:
+            selector: A TCSS selector.
+            expect_type: Expected type, or `None` for any DOMNode.
+
+        Raises:
+            InvalidQueryFormat: If the selector is invalid.
+            NoMatches: If there are no matching ancestors.
+
+        Returns:
+            A DOMNode or subclass if `expect_type` is provided.
+        """
+        base_node = self._get_dom_base()
+        if isinstance(selector, str):
+            query_selector = selector
+        else:
+            query_selector = selector.__name__
+
+        try:
+            selector_set = parse_selectors(query_selector)
+        except TokenError:
+            raise InvalidQueryFormat(
+                f"Unable to parse {query_selector!r} as a query; check for syntax errors"
+            ) from None
+        if base_node.parent is not None:
+            for node in base_node.parent.ancestors_with_self:
+                if not match(selector_set, node):
+                    continue
+                if expect_type is not None and not isinstance(node, expect_type):
+                    continue
+                return node
+        raise NoMatches(f"No ancestor matches {selector!r} on {self!r}")
 
     def set_styles(self, css: str | None = None, **update_styles: Any) -> Self:
         """Set custom styles on this object.
@@ -1343,9 +1673,9 @@ class DOMNode(MessagePump):
             Self.
         """
         if add:
-            self.add_class(*class_names, update=update)
+            self.add_class(*class_names, update=update and self.is_attached)
         else:
-            self.remove_class(*class_names, update=update)
+            self.remove_class(*class_names, update=update and self.is_attached)
         return self
 
     def set_classes(self, classes: str | Iterable[str]) -> Self:
@@ -1435,7 +1765,10 @@ class DOMNode(MessagePump):
         Returns:
             `True` if the DOM node has the pseudo class, `False` if not.
         """
-        return class_name in self.get_pseudo_classes()
+        try:
+            return self._PSEUDO_CLASSES[class_name](self)
+        except KeyError:
+            return False
 
     def has_pseudo_classes(self, class_names: set[str]) -> bool:
         """Check the node has all the given pseudo classes.
@@ -1446,13 +1779,47 @@ class DOMNode(MessagePump):
         Returns:
             `True` if all pseudo class names are present.
         """
-        return class_names.issubset(self.get_pseudo_classes())
+        PSEUDO_CLASSES = self._PSEUDO_CLASSES
+        try:
+            return all(PSEUDO_CLASSES[name](self) for name in class_names)
+        except KeyError:
+            return False
+
+    @property
+    def _pseudo_classes_cache_key(self) -> tuple[int, ...]:
+        """A cache key used when updating a number of nodes from the stylesheet."""
+        return ()
 
     def refresh(
         self, *, repaint: bool = True, layout: bool = False, recompose: bool = False
     ) -> Self:
         return self
 
+    def check_action(self, action: str, parameters: tuple[object, ...]) -> bool | None:
+        """Check whether an action is enabled.
+
+        Implement this method to add logic for [dynamic actions](/guide/actions#dynamic-actions) / bindings.
+
+        Args:
+            action: The name of an action.
+            parameters: A tuple of any action parameters.
+
+        Returns:
+            `True` if the action is enabled+visible,
+                `False` if the action is disabled+hidden,
+                `None` if the action is disabled+visible (grayed out in footer)
+        """
+        return True
+
+    def refresh_bindings(self) -> None:
+        """Call to prompt widgets such as the [Footer][textual.widgets.Footer] to update
+        the display of key bindings.
+
+        See [actions](/guide/actions#dynamic-actions) for how to use this method.
+
+        """
+        self.screen.refresh_bindings()
+
     async def action_toggle(self, attribute_name: str) -> None:
         """Toggle an attribute on the node.
 
